#game entities - pacdog, cats and catnip-pellets

import pygame
import random

from pykitten.kittens import Kitten, Chara

from ai import PacAi
from res import GFX



class PacDog(Chara):
    def __init__(self):
        class Stacky(list):
            max = 1
            def append(self, item):
                if len(self) >= self.max:
                    self.pop(0)
                list.append(self, item)

        Chara.__init__(self, GFX["pacdog"])
        self.next_dir = None
        self.scary = 0
        self.speed = 2
        self.coord = (0,0)
        self.ai = PacAi(self)
        self.next_vel = None
        self.selected = 0

    def go(self, vel):
        self.set_vel((vel[0] * self.speed, vel[1] * self.speed))

    def set_scary(self, val):
        if val:
            self.scary = 1
            self.imglists = GFX["dpacdog"]
        else:
            self.scary = 0
            self.imglists = GFX["pacdog"]

        self.set_images(self.imglists[self.cur_dir])

    def test_move(self, dir):
        if dir==0:  #down
            vel = (0, self.speed)
        elif dir==1:  #up
            vel = (0, -self.speed)
        elif dir==2:  #left
            vel = (-self.speed, 0)
        else:  #right
            vel = (self.speed, 0)

        self.set_pos((self.pos[0]+vel[0],self.pos[1]+vel[1]))

    def vel2dir(self,vel):
        if vel[1] > 0 :
            dir = 0
        elif vel[1] < 0:
            dir = 1
        elif vel[0] < 0:
            dir = 2
        else:
            dir = 3
        return dir

    def update(self):
        Chara.update(self)
        if self.selected:
            img = GFX["glowdog"][self.cur_dir].copy()
            img.blit(self.image, (0,0))
            self.set_image(img)



class Cat(Kitten):
    eye_pos = (7,8)
    ai = False
    def __init__(self, nr):
        key = "g"+str(nr)
        if key in GFX.keys():
            img = GFX[key]
        else:
            img = GFX["g0"]
        Kitten.__init__(self, img)

        self.nr = nr
        self.base_img = img.copy()
        self.next_vel = None
        self.coord = (0,0)
        self.scary = 1
        self.speed = 2
        self.selected = 0
        self.simg = img.copy()
        self.simg.blit(GFX["catglow"], (0,0))
        self.selected = 0

    def go(self, vel):
        self.set_vel((vel[0] * self.speed, vel[1] * self.speed))

    def update(self):
        def draw_eyes(eyes):
            self.image.blit(img, (0,0))
            self.image.blit(GFX[eyes], self.eye_pos)

        Kitten.update(self)
        img = self.image
        if self.selected:
            img = self.simg

        if self.vel[1] < 0:
            draw_eyes("up")
        elif self.vel[1] > 0:
            draw_eyes("down")
        elif self.vel[0] < 0:
            draw_eyes("left")
        elif self.vel[0] > 0:
            draw_eyes("right")

    def deselect(self):
        self.selected = 0
        self.image = self.base_img.copy()

    def select(self):
        self.selected = 1
        self.image = self.simg

    def set_scary(self, val):
        self.scary = val

    def set_movedir(self, dir):
        pass
    def vel2dir(self, vel):
        return 0

class CatNip(Kitten):
    def __init__(self, coord):
        Kitten.__init__(self, GFX["catnip"])
        self.coord = coord


class CatAway(Kitten):
    def __init__(self, coord):
        Kitten.__init__(self, GFX["cataway"])
        self.coord = coord


