import os
import pygame

from pykitten.data import load, filepath


from res import GFX


def get_maps(folder="lvl"):
    path = filepath(folder)
    return os.listdir(path)

def load_map(filename, folder):
    try:
        f = load(folder+"/"+filename, "r")
        f.close()
        return 0
    except:
        print "Could not load map", filename
        return 1

class Map(pygame.Surface):
    def __init__(self, filename="test.map", edit=False, folder="lvl"):
        self.max = (24,17)

        #load mapdata
        self.mapdata = {}
        self.ghost_start = []
        self.pac_start = []
        self.cataway = []
        self.pellets = []
        self.walls = []
        self.ground = []
        self.orig_map = {}
        self.folder = folder

        #self.folder = "custom"

        try:
            f = load(self.folder+"/"+filename, "r")
            x, y = 0, 0
            for line in f:
                x = 0
                line.strip()

                for t in line:
                    if t=="\n":
                        continue
                    elif t=="1":
                        self.mapdata[x,y] = 1
                        self.walls.append((x,y))
                    else:
                        self.mapdata[x,y] = 0
                        self.ground.append((x,y))

                    if t=="G": #ghost start position
                        self.ghost_start.append((x,y))
                    elif t=="P": #pacman start position
                        self.pac_start.append((x,y))
                    elif t=="C": #cataway position
                        self.cataway.append((x,y))
                    elif t=="0":
                        self.pellets.append((x,y))

                    if edit:
                        self.orig_map[x,y] = t
                    x += 1
                y += 1

            f.close()

        except:
            print "Error loading map", filename," - using empty dummy map"
            self.mapdata = {}
            self.ghost_start = []
            self.pac_start = []
            self.cataway = []
            self.pellets = []
            self.walls = []
            self.ground = []
            siz = (12, 9)
            siz = self.max
            for y in xrange(siz[1]):
                for x in xrange(siz[0]):
                    self.mapdata[x,y] = 0
                    self.orig_map[x,y] = "2"
                    self.ground.append((x,y))
            x, y = siz


        self.csize = (x, y)
        self.size = (x*32, y*32)
        pygame.Surface.__init__(self, self.size)
        self.draw_level()




    def draw_level(self):
        self.fill((0,0,0,255))

        gfx_keys = GFX.keys()

        for key in self.mapdata.keys():
            type = self.mapdata[key]

            kx, ky = key
            x, y = key[0]*32, key[1]*32

            if type==1:
                #draw wall tile

                #check up, down, left, right tile for walls
                check = ((kx, ky-1),(kx, ky+1),(kx-1, ky),(kx+1, ky))

                adj = []
                for c in check:
                    wall = 0
                    if c in self.mapdata.keys() and self.mapdata[c]==1:
                        wall = 1
                    adj.append(wall)

                #cludge: convert adj-list to tuple for adj_dict
                adj = (adj[0], adj[1], adj[2], adj[3])

                adj_dict = {
                    (0,0,0,0): "dot",
                    (1,1,1,1): "cross",

                    (0,0,1,1): "long",
                    (1,1,0,0): "long1",

                    (1,0,0,1): "corner",
                    (1,0,1,0): "corner1",
                    (0,1,0,1): "corner2",
                    (0,1,1,0): "corner3",

                    (0,0,0,1): "end",
                    (0,0,1,0): "end1",
                    (1,0,0,0): "end2",
                    (0,1,0,0): "end3",

                    (1,0,1,1): "tjunct",
                    (0,1,1,1): "tjunct1",
                    (1,1,1,0): "tjunct2",
                    (1,1,0,1): "tjunct3",
                }

                if adj in adj_dict.keys():
                    tile = adj_dict[adj]
                else:
                    tile = "dot"

                self.blit(GFX[tile], (x,y))


    def get_blocking(self):
        block = []
        for k in self.mapdata.keys():
            if self.mapdata[k]==1:
                block.append(k)
        return block


    def draw_items(self):
        tiles = {
            "0": "catnip",
            "G": "ghost",
            "P": "pacdog-img",
            "C": "cataway",
        }

        self.draw_level()
        for k in self.orig_map.keys():
            t = self.orig_map[k]
            if t != "1" and t != "2":
                self.blit(GFX[tiles[t]], (k[0]*32, k[1]*32))

    def put_tile(self, name, coord):
        tiles = {
            "catnip": "0",
            "ghost": "G",
            "pacdog-img": "P",
            "cataway": "C",
            "corner": "1",
            "ground": "2",
        }

        t = tiles[name]
        self.orig_map[coord] = t
        if t=="1":
            self.mapdata[coord] = 1
        else:
            self.mapdata[coord] = 0


    def resize(self, val):
        cx, cy = self.csize
        if val=="x+" and cx+1 <= self.max[0]:
            for i in xrange(cy):
                k = (cx, i)
                self.mapdata[k] = 0
                self.orig_map[k] = "2"
            cx += 1
        elif val=="y+" and cy+1 <= self.max[1]:
            for i in xrange(cx):
                k = (i,cy)
                self.mapdata[k] = 0
                self.orig_map[k] = "2"
            cy += 1
        elif val=="x-" and cx-1 > 0:
            cx -= 1
            for i in xrange(cy):
                k = (cx, i)
                if self.mapdata.has_key(k) and self.orig_map.has_key(k):
                    self.mapdata.pop(k)
                    self.orig_map.pop(k)
        elif val=="y-" and cy-1 > 0:
            cy -= 1
            for i in xrange(cx):
                k = (i, cy)
                if self.mapdata.has_key(k) and self.orig_map.has_key(k):
                    self.mapdata.pop(k)
                    self.orig_map.pop(k)

        self.csize = (cx, cy)
        self.size = (cx*32, cy*32)
        pygame.Surface.__init__(self, self.size)


    def save(self, filename, folder="custom"):
        try:
            f = load(folder+"/"+filename, "w")
            for h in xrange(self.csize[1]):
                line = ""
                for w in xrange(self.csize[0]):
                    if self.orig_map.has_key((w,h)):
                        line = line + self.orig_map[w,h]
                f.write(line+"\n")
            f.close()
        except:
            print "Could not save map to", filename
            return 1


    def load(self, filename):
        try:
            f = load(self.folder+"/"+filename, "r")
            f.close()
            return 0
        except:
            print "Could not load map", filename
            return 1

    def get_score(self):
        return len(self.pellets)*10 + len(self.cataway)*50
